@@ -1,5 +1,6 @@
 from invoke import task
 
-@task(aliases=('z', 'a'))
+
+@task(aliases=("z", "a"))
 def toplevel(c):
     pass
