@@ -2,8 +2,7 @@ import pickle
 import os
 from os.path import join
 
-from invoke.util import six
-from mock import patch, call, Mock
+from unittest.mock import patch, call, Mock
 import pytest
 from pytest_relaxed import raises
 
@@ -274,12 +273,7 @@ Valid real attributes: ['clear', 'clone', 'env_prefix', 'file_prefix', 'from_dat
             assert c == c2
             assert len(c) == 1
             assert c.get("foo") == "bar"
-            if six.PY2:
-                assert c.has_key("foo") is True  # noqa
-                assert list(c.iterkeys()) == ["foo"]
-                assert list(c.itervalues()) == ["bar"]
             assert list(c.items()) == [("foo", "bar")]
-            assert list(six.iteritems(c)) == [("foo", "bar")]
             assert list(c.keys()) == ["foo"]
             assert list(c.values()) == ["bar"]
 
@@ -735,21 +729,10 @@ Valid real attributes: ['clear', 'clone', 'env_prefix', 'file_prefix', 'from_dat
 
         class type_casting:
             def strings_replaced_with_env_value(self):
-                os.environ["INVOKE_FOO"] = u"myvalue"
+                os.environ["INVOKE_FOO"] = "myvalue"
                 c = Config(defaults={"foo": "myoldvalue"})
                 c.load_shell_env()
-                assert c.foo == u"myvalue"
-                assert isinstance(c.foo, six.text_type)
-
-            def unicode_replaced_with_env_value(self):
-                # Python 3 doesn't allow you to put 'bytes' objects into
-                # os.environ, so the test makes no sense there.
-                if six.PY3:
-                    return
-                os.environ["INVOKE_FOO"] = "myunicode"
-                c = Config(defaults={"foo": u"myoldvalue"})
-                c.load_shell_env()
-                assert c.foo == "myunicode"
+                assert c.foo == "myvalue"
                 assert isinstance(c.foo, str)
 
             def None_replaced(self):
@@ -782,12 +765,7 @@ Valid real attributes: ['clear', 'clone', 'env_prefix', 'file_prefix', 'from_dat
                 tests = [
                     (int, "5", 5),
                     (float, "5.5", 5.5),
-                    # TODO: more?
                 ]
-                # Can't use '5L' in Python 3, even having it in a branch makes
-                # it upset.
-                if not six.PY3:
-                    tests.append((long, "5", long(5)))  # noqa
                 for old, new_, result in tests:
                     os.environ["INVOKE_FOO"] = new_
                     c = Config(defaults={"foo": old()})
@@ -797,7 +775,7 @@ Valid real attributes: ['clear', 'clone', 'env_prefix', 'file_prefix', 'from_dat
             def arbitrary_types_work_too(self):
                 os.environ["INVOKE_FOO"] = "whatever"
 
-                class Meh(object):
+                class Meh:
                     def __init__(self, thing=None):
                         pass
 
@@ -1069,7 +1047,7 @@ Valid real attributes: ['clear', 'clone', 'env_prefix', 'file_prefix', 'from_dat
                 else:
                     assert False, "Non-class obj did not raise TypeError!"
 
-                class Foo(object):
+                class Foo:
                     pass
 
                 try:
