@@ -1,18 +1,23 @@
 import json
 import os
 import sys
+from io import BytesIO
 
-from invoke.util import six
-from mock import patch, Mock, ANY
+from invoke.util import Lexicon
+from unittest.mock import patch, Mock, ANY
 import pytest
 from pytest import skip
 from pytest_relaxed import trap
 
 from invoke import (
+    Argument,
     Collection,
     Config,
     Executor,
+    Exit,
     FilesystemLoader,
+    ParserContext,
+    ParseResult,
     Program,
     Result,
     Task,
@@ -20,6 +25,7 @@ from invoke import (
 )
 from invoke import main
 from invoke.util import cd
+from invoke.config import merge_dicts
 
 from _util import (
     ROOT,
@@ -242,6 +248,21 @@ class Program_:
             klass.assert_called_with(ANY, ANY, ANY)
             klass.return_value.execute.assert_called_with(ANY)
 
+        def executor_class_may_be_overridden_via_configured_string(self):
+            class ExecutorOverridingConfig(Config):
+                @staticmethod
+                def global_defaults():
+                    defaults = Config.global_defaults()
+                    path = "custom_executor.CustomExecutor"
+                    merge_dicts(defaults, {"tasks": {"executor_class": path}})
+                    return defaults
+
+            mock = load("custom_executor").CustomExecutor
+            p = Program(config_class=ExecutorOverridingConfig)
+            p.run("myapp noop", exit=False)
+            assert mock.assert_called
+            assert mock.return_value.execute.called
+
         def executor_is_given_access_to_core_args_and_remainder(self):
             klass = Mock()
             cmd = "myapp -e foo -- myremainder"
@@ -261,6 +282,65 @@ class Program_:
             # Also make sure it's a list for easier tweaking/appending
             assert isinstance(core_args, list)
 
+    class args_property:
+        def shorthand_for_self_core_args(self):
+            "is shorthand for self.core[0].args"
+            p = Program()
+            p.run("myapp -e noop", exit=False)
+            args = p.args
+            assert isinstance(args, Lexicon)
+            assert args.echo.value is True
+
+    class core_args_from_task_contexts:
+        # NOTE: many of these use Program.args in lieu of Program.core[0], for
+        # convenience, tho also because initially the behavior was _in_ .args
+        def core_context_gets_updated_with_core_flags_from_tasks(self):
+            # Part of #466.
+            p = Program()
+            p.run("myapp -e noop --hide both", exit=False)
+            # Was given in core
+            assert p.args.echo.value is True
+            # Was given in per-task
+            assert p.args.hide.value == "both"
+
+        def copying_from_task_context_does_not_set_empty_list_values(self):
+            # Less of an issue for scalars, but for list-type args, doing
+            # .value = <default value> actually ends up creating a
+            # list-of-lists.
+            p = Program()
+            # Set up core-args parser context with an iterable arg that hasn't
+            # seen any value yet
+            def filename_args():
+                return [Argument("filename", kind=list)]
+
+            p.core = ParseResult([ParserContext(args=filename_args())])
+            # And a core-via-tasks context with a copy of that same arg, which
+            # also hasn't seen any value yet
+            p.core_via_tasks = ParserContext(args=filename_args())
+            # Now the behavior of .args can be tested as desired
+            assert p.args["filename"].value == []  # Not [[]]!
+
+        def copying_from_task_context_does_not_overwrite_good_values(self):
+            # Another subcase, also mostly applying to list types: core context
+            # got a useful value, nothing was found in the per-task context;
+            # when a naive 'is not None' check is used, this overwrites the
+            # good value with an empty list.
+            # (Other types tend to not have this problem because their ._value
+            # is always None when not set. TODO: maybe this should be
+            # considered incorrect behavior for list type args?)
+            def make_arg():
+                return Argument("filename", kind=list)
+
+            p = Program()
+            # Core arg, which got a value
+            arg = make_arg()
+            arg.value = "some-file"  # appends to list
+            p.core = ParseResult([ParserContext(args=[arg])])
+            # Set core-via-tasks version to vanilla/blank/empty-list version
+            p.core_via_tasks = ParserContext(args=[make_arg()])
+            # Call .args, expect that the initial value was not overwritten
+            assert p.args.filename.value == ["some-file"]
+
     class run:
         # NOTE: some of these are integration-style tests, but they are still
         # fast tests (so not needing to go into the integration suite) and
@@ -354,9 +434,7 @@ class Program_:
             # Expect repr() of exception prints to stderr
             # NOTE: this partially duplicates a test in runners.py; whatever.
             stderr = sys.stderr.getvalue()
-            assert (
-                stderr
-                == """Encountered a bad command exit code!
+            expected = """Encountered a bad command exit code!
 
 Command: 'meh'
 
@@ -371,7 +449,7 @@ Stderr:
 ohnoz!
 
 """
-            )
+            assert stderr == expected
             # And exit with expected code (vs e.g. 1 or 0)
             mock_exit.assert_called_with(54)
 
@@ -383,8 +461,8 @@ ohnoz!
                 Result(
                     command="meh",
                     exited=54,
-                    stdout=u"this is not ascii: \u1234",
-                    stderr=u"this is also not ascii: \u4321",
+                    stdout="this is not ascii: \u1234",
+                    stderr="this is also not ascii: \u4321",
                     encoding="utf-8",
                     hide=("stdout", "stderr"),
                 )
@@ -410,9 +488,44 @@ Stderr:
 this is also not ascii: \xe4\x8c\xa1
 
 """
-            got = six.BytesIO.getvalue(sys.stderr)
+            got = BytesIO.getvalue(sys.stderr)
             assert got == expected
 
+        class Exit_:
+            @patch("invoke.program.sys.exit")
+            def defaults_to_exiting_0(self, mock_exit):
+                p = Program()
+                p.execute = Mock(side_effect=Exit())
+                p.run("myapp foo")
+                mock_exit.assert_called_once_with(0)
+
+            @trap
+            @patch("invoke.program.sys.exit")
+            def prints_message_exiting_1_if_message_given(self, mock_exit):
+                p = Program()
+                p.execute = Mock(side_effect=Exit("onoz"))
+                p.run("myapp foo")
+                mock_exit.assert_called_once_with(1)
+                assert sys.stderr.getvalue() == "onoz\n"
+
+            @trap
+            @patch("invoke.program.sys.exit")
+            def may_explicitly_supply_code_with_message(self, mock_exit):
+                p = Program()
+                p.execute = Mock(side_effect=Exit("onoz", code=17))
+                p.run("myapp foo")
+                mock_exit.assert_called_once_with(17)
+                assert sys.stderr.getvalue() == "onoz\n"
+
+            @trap
+            @patch("invoke.program.sys.exit")
+            def may_explicitly_supply_code_without_message(self, mock_exit):
+                p = Program()
+                p.execute = Mock(side_effect=Exit(code=17))
+                p.run("myapp foo")
+                mock_exit.assert_called_once_with(17)
+                assert sys.stderr.getvalue() == ""
+
         def should_show_core_usage_on_core_parse_failures(self):
             skip()
 
@@ -479,6 +592,9 @@ Core options:
   -p, --pty                          Use a pty when executing shell commands.
   -r STRING, --search-root=STRING    Change root directory used for finding
                                      task modules.
+  -R, --dry                          Echo commands instead of running.
+  -T INT, --command-timeout=INT      Specify a global command execution
+                                     timeout, in seconds.
   -V, --version                      Show version and exit.
   -w, --warn-only                    Warn, instead of failing, when shell
                                      commands fail.
@@ -1077,7 +1193,7 @@ Default 'build' task: .all
                     )
 
             class json:
-                def setup(self):
+                def setup_method(self):
                     # Stored expected data as an actual JSON file cuz it's big
                     # & looks like crap if inlined. Plus by round-tripping it
                     # we remove the pretty-printing. Win-win?
@@ -1144,6 +1260,13 @@ Default 'build' task: .all
         def echo(self):
             self._test_flag("-e", "echo")
 
+        def timeout(self):
+            for flag in ("-T", "--command-timeout"):
+                p = Program()
+                p.execute = Mock()  # neuter
+                p.run("inv {} 5 foo".format(flag))
+                assert p.config.timeouts.command == 5
+
     class configuration:
         "Configuration-related concerns"
 
